#define O_ALTWERASE 0
#define O_AUTOINDENT 1
#define O_AUTOPRINT 2
#define O_AUTOWRITE 3
#define O_BACKUP 4
#define O_BEAUTIFY 5
#define O_CDPATH 6
#define O_CEDIT 7
#define O_COLUMNS 8
#define O_COMMENT 9
#define O_DIRECTORY 10
#define O_EDCOMPATIBLE 11
#define O_ESCAPETIME 12
#define O_ERRORBELLS 13
#define O_EXRC 14
#define O_EXTENDED 15
#define O_FILEC 16
#define O_FLASH 17
#define O_HARDTABS 18
#define O_ICLOWER 19
#define O_IGNORECASE 20
#define O_KEYTIME 21
#define O_LEFTRIGHT 22
#define O_LINES 23
#define O_LISP 24
#define O_LIST 25
#define O_LOCKFILES 26
#define O_MAGIC 27
#define O_MATCHTIME 28
#define O_MESG 29
#define O_MODELINE 30
#define O_MSGCAT 31
#define O_NOPRINT 32
#define O_NUMBER 33
#define O_OCTAL 34
#define O_OPEN 35
#define O_OPTIMIZE 36
#define O_PARAGRAPHS 37
#define O_PATH 38
#define O_PRINT 39
#define O_PROMPT 40
#define O_READONLY 41
#define O_RECDIR 42
#define O_REDRAW 43
#define O_REMAP 44
#define O_REPORT 45
#define O_RULER 46
#define O_SCROLL 47
#define O_SEARCHINCR 48
#define O_SECTIONS 49
#define O_SECURE 50
#define O_SHELL 51
#define O_SHELLMETA 52
#define O_SHIFTWIDTH 53
#define O_SHOWMATCH 54
#define O_SHOWMODE 55
#define O_SIDESCROLL 56
#define O_SLOWOPEN 57
#define O_SOURCEANY 58
#define O_TABSTOP 59
#define O_TAGLENGTH 60
#define O_TAGS 61
#define O_TERM 62
#define O_TERSE 63
#define O_TILDEOP 64
#define O_TIMEOUT 65
#define O_TTYWERASE 66
#define O_VERBOSE 67
#define O_W1200 68
#define O_W300 69
#define O_W9600 70
#define O_WARN 71
#define O_WINDOW 72
#define O_WINDOWNAME 73
#define O_WRAPLEN 74
#define O_WRAPMARGIN 75
#define O_WRAPSCAN 76
#define O_WRITEANY 77
#define O_OPTIONCOUNT 78
